"""
A simple to use Python Library used to handle API Calls for all kinds of unique discord bots.

:copyright: (c) 2021-present HiddenStorm
:license: MIT, see LICENSE for more details.
"""

__name__ = "DiscAPI"
__version__ = "0.2.0a"
__author__ = "HiddenStorm"

from .Client import Client
from .Guild import Guild
